function dic(wrd,opt)
%Dictionary for MATLAB by jph in combination with
%the GER-ENG.TXT file provided under
%http://wftp.tu-chemnitz.de/pub/Local/urz/ding/ger-eng
%
%(!) FIRST TIME ONLY: download the file ger-eng.txt
%(!) from the url above in your MATLAB/WORK folder and in
%(!) MATLAB call
%                 dic #setup
% 
%this creates a fast access database for future inquiry.
%
%regular usage:
%                 dic EXPR [OPT]
%
%EXPR: german or english word beginning/fragment
%      examples: EXPR=ragme, EXPR='carrion eater'
%OPT : concatenation of optional search specifications
%  a - display all results; i.e. do not truncate at 15
%  c - cASe sensitive search
%  d - classify EXPR as "Deutsch" 
%  e - classify EXPR as English
%  m - string found as a fragment is a match, otherwise
%      looks only for work beginnings
%  s - wide split
%  w - find EXPR as whole word (a priori case insensitive)
%
%example:
%                 dic ragme em
%results in
%
% Bruchstck {n}; FRAGMEnt {n} ...............  fRAGMEnt
% | Bruchstcke {pl}; FRAGMEnte {pl}            | fRAGMEnts
% Knochensplitter {m}                           bone fRAGMEnt
% Lumpenhndler {m}                             ragman
% | Lumpenhndler {pl}                          | RAGMEn
% Lumpensammler {m} ..........................  ragman
% | Lumpensammler {pl}                          | RAGMEn
% MassenfRAGMEnt {n}                            mass fRAGMEnt
% Scherbe {f}                                   broken piece; fRAGMEnt
% | Scherben {pl}                               | broken fRAGMEnts
% Scherbenhaufen {m} .........................  pile of broken RAGMEnts
% Unvollstndigkeit {f}                         fRAGMEntariness
% Zerstckelung {f}; Zerstckeln {n}; Zersplit  fRAGMEntation
% | Zerstckelungen {pl}                        | fRAGMEntations
% beseitigt die Aufsplitterung                  unfRAGMEnts
% fRAGMEntarisch {adv} .......................  fRAGMEntarily
% fRAGMEntieren                                 to fRAGMEnt
% | fRAGMEntierend                              | fRAGMEnting
% | fRAGMEntiert                                | fRAGMEnted
% zerstckelt                                   fRAGMEnted
% zusammensetzen .............................  to unfRAGMEnt

if ~nargin;	help dic;	return; end

if strcmp(wrd,'#setup')
	disp('preparing database... may take up to one minute.')
	fid=fopen('ger-eng.txt','r');
	if fid<0; error('file not found.'); end;
	str=blanks(7000000);
	lin=fgetl(fid);
	lnc=0; mln=0; cnt=zeros(1,1600);
	ptr=1;
	while ischar(lin);
		lln=length(lin);
		if lln
			if lin(1)~='#'
				cnt(lln)=cnt(lln)+1;				
				%lin=lin(1:min(lln,300));
				%lln=length(lin);
				str(ptr+[1:lln+2])=[' ' lin];
				ptr=ptr+lln+2;				
				lnc=lnc+1;
			end
		end	
		lin=fgetl(fid);
	end;
	str=str(1:ptr+1);
	str(ptr+1)='';
	fclose(fid);
	bar(1:1600,cnt);
	save 'ger-eng' str	
	return
end

load 'ger-eng'
if ~exist('opt'); opt=''; end
if ~ismember('m',opt); wrd=[' ' wrd]; end
if ~ismember('c',opt);
	f=findstr(lower(wrd),lower(str));
else
	f=findstr(wrd,str);	
end

MAXR=Inf;
SPLIT=45*(1+ismember('s',opt));
if ~ismember('a',opt); MAXR=25; end
cnt=0; lst=0; out=0;
while cnt<length(f)&out<=MAXR;
	cnt=cnt+1;
	c1=f(cnt);
	if c1>lst
		bdl=c1; bdr=c1;
		while str(bdl)~=''; bdl=bdl-1; end
		while str(bdr)~=''; bdr=bdr+1; end
		lst=bdr;
		rsp=[str(bdl+1:bdr-1) ' '];
		d=findstr(rsp,'::');
		if ~isempty(d)
			d=d(1);
			deu=0; eng=0; fnd=0;
			for c2=findstr(lower(rsp),lower(wrd));
				if ~ismember('w',opt)|ismember(rsp(c2+length(wrd)),' ;');
					fnd=1;
					if c2<d; deu=1; end
					if c2>d; eng=1; end
					rsp(c2+[0:length(wrd)-1])=upper(wrd);
				end
			end
			if fnd&(~ismember('d',opt)|deu)&(~ismember('e',opt)|eng)
				if ~out; disp(' '); end				
				r{1}=rsp(1:d-1); r{2}=rsp(d+2:end);
				for c3=1:2
					[h{c3},r{c3}]=strtok(r{c3},'|');
					h{c3}=[h{c3} jidic(out,c3,SPLIT)];
					h{c3}=h{c3}(1:SPLIT);
				end
				disp([h{1} ' ' h{2}])
				out=out+1;
				while ~(isempty(r{1})&isempty(r{2}))					
					for c3=1:2
						[h{c3},r{c3}]=strtok(r{c3},'|');
						h{c3}=[h{c3} jidic(out,c3,SPLIT)];
						h{c3}=h{c3}(1:SPLIT-2);
					end					
					disp([' |' h{1} '  |' h{2}])
					out=out+1;
				end
				%disp(rsp(1:d-1))
				%disp(['  \-' rsp(d+2:end)])
			end
		else
			rsp
		end
	end
end				
if out>MAXR; fprintf('  ___ truncated ___'); end
if ~out; disp('no matches.'); end

function del=jidic(out,c3,SPLIT)
	if mod(out,5)|c3==2
		del=blanks(SPLIT);
	else
		del=char(repmat(46,[1 SPLIT]));
	end